package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.nachrichten.common.vorgang.GrpcServiceKontoTestFactory;
import de.ozgcloud.nachrichten.common.vorgang.Vorgang;
import de.ozgcloud.nachrichten.common.vorgang.VorgangTestFactory;
import de.ozgcloud.nachrichten.postfach.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory;

class RueckfrageMapperTest {

	private final RueckfrageMapper mapper = Mappers.getMapper(RueckfrageMapper.class);

	@DisplayName("From postfachNachricht")
	@Nested
	class TestFromPostfachNachricht {

		@Test
		void shouldMapId() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getId()).isEqualTo(PostfachNachrichtTestFactory.ID);
		}

		@Test
		void shouldMapVorgangId() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getVorgangId()).isEqualTo(MessageTestFactory.VORGANG_ID);
		}

		@Test
		void shouldMapPostfachAddress() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getPostfachAddress()).usingRecursiveComparison().isEqualTo(PostfachAddressTestFactory.create());
		}

		@Test
		void shouldMapSentAt() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getSentAt()).isEqualTo(PostfachNachrichtTestFactory.SENT_AT);
		}

		@Test
		void shouldHaveDefaultStatus() {
			var mapped = mapFromPostfachNachricht();

			assertThat(mapped.getStatus()).isEqualTo(RueckfrageStatus.NEW);
		}

		private RueckfrageHead mapFromPostfachNachricht() {
			return mapper.fromPostfachNachricht(PostfachNachrichtTestFactory.create());
		}
	}

	@DisplayName("To rueckfrage")
	@Nested
	class TestToRueckfrage {

		private final Vorgang vorgang = VorgangTestFactory.create();
		private final PostfachNachricht rueckfrageNachricht = PostfachNachrichtTestFactory.create();

		public static final String ANSWERED_AT_STR = "2020-04-01T11:30:10Z";
		public static final ZonedDateTime ANSWERED_AT_AT = ZonedDateTime.parse(ANSWERED_AT_STR);
		private final Optional<ZonedDateTime> firstSentAt = Optional.of(ANSWERED_AT_AT);

		@Test
		void shouldMapId() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getId()).isEqualTo(PostfachNachrichtTestFactory.ID);
		}

		@Test
		void shouldMapText() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getText()).isEqualTo(PostfachNachrichtTestFactory.MAIL_BODY);
		}

		@Test
		void shouldMapAttachments() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getAttachmentFileIdList()).containsExactly(PostfachNachrichtTestFactory.ATTACHMENT_FILE_ID);
		}

		@Test
		void shouldMapVorgangName() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getVorgangName()).isEqualTo(VorgangTestFactory.NAME);
		}

		@Test
		void shouldMapVorgangNummer() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getVorgangNummer()).isEqualTo(VorgangTestFactory.NUMMER);
		}

		@Test
		void shouldMapTrustLevel() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getTrustLevel()).isEqualTo(GrpcServiceKontoTestFactory.TRUST_LEVEL);
		}

		@Test
		void shouldMapAnsweredAt() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getAnsweredAt()).isEqualTo(ANSWERED_AT_STR);
		}

		@Test
		void shouldMapStatus() {
			var rueckfrage = toRueckfrage();

			assertThat(rueckfrage.getStatus()).isEqualTo(RueckfrageStatus.ANSWERED.name());
		}

		private GrpcRueckfrage toRueckfrage() {
			return mapper.toRueckfrage(firstSentAt, vorgang, rueckfrageNachricht);
		}
	}
}
