/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.bayernid;

import static de.ozgcloud.nachrichten.postfach.bayernid.BayernIdAttachmentTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.mongodb.client.gridfs.model.GridFSFile;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.nachrichten.postfach.BinaryFileService;
import de.ozgcloud.nachrichten.postfach.FileId;

class BayernIdAttachmentServiceTest {

	private static final FileId FILE_ID = FileId.from("42");

	@InjectMocks
	@Spy
	private BayernIdAttachmentService bayernIdAttachmentService;

	@Mock
	private BinaryFileService fileService;

	@Mock
	private GridFSFile gridFsfile;

	@Mock
	private Document metadata;

	@Nested
	class TestLoadingAttachment {

		@BeforeEach
		void init() {
			when(gridFsfile.getMetadata()).thenReturn(metadata);
			when(fileService.getFile(any())).thenReturn(gridFsfile);
		}

		@Test
		void shouldCallGetFile() {
			when(fileService.getUploadedFileStream(any())).thenReturn(new ByteArrayInputStream(CONTENT));

			bayernIdAttachmentService.getMessageAttachment(FILE_ID);

			verify(fileService).getFile(FILE_ID);
		}

		@Test
		void shouldCallGetAttachmentContent() {
			when(fileService.getUploadedFileStream(any())).thenReturn(new ByteArrayInputStream(CONTENT));

			bayernIdAttachmentService.getMessageAttachment(FILE_ID);

			verify(bayernIdAttachmentService).getMessageAttachment(FILE_ID);
		}

		@Test
		void shouldCallBuildBayernIdAttachment() {
			var contentStream = getContentStream();
			doReturn(contentStream).when(bayernIdAttachmentService).getAttachmentContentStream(any());

			bayernIdAttachmentService.getMessageAttachment(FILE_ID);

			verify(bayernIdAttachmentService).buildBayernIdAttachment(metadata, contentStream);
		}

	}

	@Nested
	class TestLoadAttachmentError {

		@Test
		@DisplayName("should throw TechnicalException if attachment not found")
		void shouldThrowException() {
			when(fileService.getFile(any())).thenReturn(null);

			assertThrows(TechnicalException.class, () -> bayernIdAttachmentService.getMessageAttachment(FILE_ID));
		}

		@Test
		@DisplayName("should throw TechnicalException if metadata is null")
		void shouldThrowExceptionIfNoMetadata() {
			when(fileService.getFile(any())).thenReturn(gridFsfile);
			when(gridFsfile.getMetadata()).thenReturn(null);

			assertThrows(TechnicalException.class, () -> bayernIdAttachmentService.getMessageAttachment(FILE_ID));
		}
	}

	@Nested
	class TestBuildBayernIdAttachment {

		@BeforeEach
		void setup() {
			when(metadata.getString(BayernIdAttachmentService.NAME_KEY)).thenReturn(BayernIdAttachmentTestFactory.FILENAME);
			when(metadata.getString(BayernIdAttachmentService.CONTENT_TYPE_KEY)).thenReturn(BayernIdAttachmentTestFactory.CONTENT_TYPE);
		}

		@Test
		void shouldSetFileName() {
			var attachment = buildBayernIdAttachment();

			assertThat(attachment.getFileName()).isEqualTo(BayernIdAttachmentTestFactory.FILENAME);
		}

		@Test
		void shouldSetContent() {
			var attachment = buildBayernIdAttachment();

			assertThat(attachment.getContent()).hasSameContentAs(new ByteArrayInputStream(CONTENT));
		}

		@Test
		void shouldSetContentType() {
			var attachment = buildBayernIdAttachment();

			assertThat(attachment.getContentType()).isEqualTo(BayernIdAttachmentTestFactory.CONTENT_TYPE);
		}

		private BayernIdAttachment buildBayernIdAttachment() {
			return bayernIdAttachmentService.buildBayernIdAttachment(metadata, getContentStream());
		}
	}

	@Nested
	class TestLoadingAttachmentContent {

		@BeforeEach
		void init() {
			when(fileService.getUploadedFileStream(any()))
					.thenReturn(new ByteArrayInputStream(CONTENT));
		}

		@Test
		void shouldGetInputStream() {
			InputStream input = bayernIdAttachmentService.getAttachmentContentStream(FILE_ID);

			assertThat(input).hasSameContentAs(getContentStream());
		}

	}
}